﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman14 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.13.12 U = G*m1*m2*(1/r2-1/r1)"; } }

 protected override string TargetVariable { get { return "U"; } }
    protected override string[] VariableNames { get { return new string[] { "m1", "m2", "r1", "r2", "G", "U"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m1", "m2", "r1", "r2", "G"}; } }

    public int Seed { get; private set; }

    public Feynman14() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman14(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var G = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();


      var U = new List<double>();

      data.Add(m1);
      data.Add(m2);
      data.Add(r1);
      data.Add(r2);
      data.Add(G);
      data.Add(U);

      for (var i = 0; i < m1.Count; i++) {
        var res = G[i] * m1[i] * m2[i] * (1 / r2[i] - 1 / r1[i]);
        U.Add(res);
      }

      return data;
    }
  }
}
