﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman15 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.14.3 U = m*g*z"; } }

 protected override string TargetVariable { get { return "U"; } }
    protected override string[] VariableNames { get { return new string[] { "m", "g", "z", "U"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m", "g", "z"}; } }

    public int Seed { get; private set; }

    public Feynman15() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman15(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var g = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var z = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var U = new List<double>();

      data.Add(m);
      data.Add(g);
      data.Add(z);
      data.Add(U);

      for (var i = 0; i < m.Count; i++) {
        var res = m[i] * g[i] * z[i];
        U.Add(res);
      }

      return data;
    }
  }
}
