﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman17 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.15.3x x1 = (x-u*t)/sqrt(1-u^2/c^2)"; } }

 protected override string TargetVariable { get { return "x1"; } }
    protected override string[] VariableNames { get { return new string[] { "x", "u", "c", "t", "x1"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"x", "u", "c", "t"}; } }

    public int Seed { get; private set; }

    public Feynman17() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman17(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var x = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 5, 10).ToList();
      var u = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 20).ToList();
      var t = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();

      var x1 = new List<double>();

      data.Add(x);
      data.Add(u);
      data.Add(c);
      data.Add(t);
      data.Add(x1);

      for (var i = 0; i < x.Count; i++) {
        var res = (x[i] - u[i] * t[i]) / Math.Sqrt(1 - Math.Pow(u[i], 2) / Math.Pow(c[i], 2));
        x1.Add(res);
      }

      return data;
    }
  }
}
