﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman18 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.15.3t t1 = (t-u*x/c^2)/sqrt(1-u^2/c^2)"; } }

 protected override string TargetVariable { get { return "t1"; } }
    protected override string[] VariableNames { get { return new string[] { "x", "c", "u", "t", "t1"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"x", "c", "u", "t"}; } }

    public int Seed { get; private set; }

    public Feynman18() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman18(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var x = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var c = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();
      var u = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var t = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var t1 = new List<double>();

      data.Add(x);
      data.Add(c);
      data.Add(u);
      data.Add(t);
      data.Add(t1);

      for (var i = 0; i < x.Count; i++) {
        var res = (t[i] - u[i] * x[i] / Math.Pow(c[i], 2)) / Math.Sqrt(1 - Math.Pow(u[i], 2) / Math.Pow(c[i], 2));
        t1.Add(res);
      }

      return data;
    }
  }
}
