﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman19 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.15.1 p = m_0*v/sqrt(1-v^2/c^2)"; } }

 protected override string TargetVariable { get { return "p"; } }
    protected override string[] VariableNames { get { return new string[] { "m_0", "v", "c", "p"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m_0", "v", "c"}; } }

    public int Seed { get; private set; }

    public Feynman19() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman19(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();

      var p = new List<double>();

      data.Add(m_0);
      data.Add(v);
      data.Add(c);
      data.Add(p);

      for (var i = 0; i < m_0.Count; i++) {
        var res = m_0[i] * v[i] / Math.Sqrt(1 - Math.Pow(v[i], 2) / Math.Pow(c[i], 2));
        p.Add(res);
      }

      return data;
    }
  }
}
