﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman20 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.16.6 v1 = (u+v)/(1+u*v/c^2)"; } }

 protected override string TargetVariable { get { return "v1"; } }
    protected override string[] VariableNames { get { return new string[] { "c", "v", "u", "v1"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"c", "v", "u"}; } }

    public int Seed { get; private set; }

    public Feynman20() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman20(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var c = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var u = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var v1 = new List<double>();

      data.Add(c);
      data.Add(v);
      data.Add(u);
      data.Add(v1);

      for (var i = 0; i < c.Count; i++) {
        var res = (u[i] + v[i]) / (1 + u[i] * v[i] / Math.Pow(c[i], 2));
        v1.Add(res);
      }

      return data;
    }
  }
}
