﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman21 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.18.4 r = (m1*r1+m2*r2)/(m1+m2)"; } }

 protected override string TargetVariable { get { return "r"; } }
    protected override string[] VariableNames { get { return new string[] { "m1", "m2", "r1", "r2", "r"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m1", "m2", "r1", "r2"}; } }

    public int Seed { get; private set; }

    public Feynman21() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman21(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var r = new List<double>();

      data.Add(m1);
      data.Add(m2);
      data.Add(r1);
      data.Add(r2);
      data.Add(r);

      for (var i = 0; i < m1.Count; i++) {
        var res = (m1[i] * r1[i] + m2[i] * r2[i]) / (m1[i] + m2[i]);
        r.Add(res);
      }

      return data;
    }
  }
}
