﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman22 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.18.12 tau = (m1*r1+m2*r2)/(m1+m2)"; } }

 protected override string TargetVariable { get { return "tau"; } }
    protected override string[] VariableNames { get { return new string[] { "r", "F", "theta", "tau"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"r", "F", "theta"}; } }

    public int Seed { get; private set; }

    public Feynman22() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman22(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var r = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var F = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 5).ToList();

      var tau = new List<double>();

      data.Add(r);
      data.Add(F);
      data.Add(theta);
      data.Add(tau);

      for (var i = 0; i < r.Count; i++) {
        var res = r[i] * F[i] * Math.Sin(theta[i]);
        tau.Add(res);
      }

      return data;
    }
  }
}
