﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman23 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.18.14 L = m*r*v*sin(theta)"; } }

 protected override string TargetVariable { get { return "L"; } }
    protected override string[] VariableNames { get { return new string[] { "m", "r", "v", "theta", "L"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m", "r", "v", "theta"}; } }

    public int Seed { get; private set; }

    public Feynman23() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman23(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var r = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var L = new List<double>();

      data.Add(m);
      data.Add(r);
      data.Add(v);
      data.Add(L);

      for (var i = 0; i < r.Count; i++) {
        var res = m[i] * r[i] * v[i] * Math.Sin(theta[i]);
        L.Add(res);
      }

      return data;
    }
  }
}
