using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman24 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.24.6 1/2*m*(omega**2+omega_0**2)*1/2*x**2"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"m", "omega", "omega_0", "x", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"m", "omega", "omega_0", "x"}; } }

    public int Seed { get; private set; }

    public Feynman24() : this((int) DateTime.Now.Ticks) { }

    public Feynman24(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data    = new List<List<double>>();
      var m       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var omega   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var omega_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();
      var x       = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 3).ToList();

      var E_n = new List<double>();

      data.Add(m);
      data.Add(omega);
      data.Add(omega_0);
      data.Add(x);
      data.Add(E_n);

      for (var i = 0; i < m.Count; i++) {
        var res = 1.0 / 2 * m[i] * (Math.Pow(omega[i], 2) + Math.Pow(omega_0[i], 2)) * 1.0 / 2 * Math.Pow(x[i], 2);
        E_n.Add(res);
      }

      return data;
    }
  }
}