using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman25 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.25.13 q/C"; } }

    protected override string TargetVariable { get { return "Volt"; } }
    protected override string[] VariableNames { get { return new[] {"q", "C", "Volt"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"q", "C"}; } }

    public int Seed { get; private set; }

    public Feynman25() : this((int) DateTime.Now.Ticks) { }

    public Feynman25(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var q    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var C    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Volt = new List<double>();

      data.Add(q);
      data.Add(C);
      data.Add(Volt);

      for (var i = 0; i < q.Count; i++) {
        var res = q[i] / C[i];
        Volt.Add(res);
      }

      return data;
    }
  }
}