using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman26 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.26.2 arcsin(n*sin(theta2))"; } }

    protected override string TargetVariable { get { return "theta1"; } }
    protected override string[] VariableNames { get { return new[] {"n", "theta2", "theta1"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "theta2"}; } }

    public int Seed { get; private set; }

    public Feynman26() : this((int) DateTime.Now.Ticks) { }

    public Feynman26(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data   = new List<List<double>>();
      var n      = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 0, 1).ToList();
      var theta2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var theta1 = new List<double>();

      data.Add(n);
      data.Add(theta2);
      data.Add(theta1);

      for (var i = 0; i < n.Count; i++) {
        var res = Math.Sinh(n[i] * Math.Sin(theta2[i]));
        theta1.Add(res);
      }

      return data;
    }
  }
}