using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman27 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.27.6 1/(1/d1+n/d2)"; } }

    protected override string TargetVariable { get { return "foc"; } }
    protected override string[] VariableNames { get { return new[] {"d1", "d2", "n", "foc"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"d1", "d2", "n"}; } }

    public int Seed { get; private set; }

    public Feynman27() : this((int) DateTime.Now.Ticks) { }

    public Feynman27(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var d1   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d2   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var n    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var foc = new List<double>();

      data.Add(d1);
      data.Add(d2);
      data.Add(n);
      data.Add(foc);

      for (var i = 0; i < d1.Count; i++) {
        var res = 1.0 / (1.0 / d1[i] + n[i] / d2[i]);
        foc.Add(res);
      }

      return data;
    }
  }
}