using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman28 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.29.4 omega/c"; } }

    protected override string TargetVariable { get { return "k"; } }
    protected override string[] VariableNames { get { return new[] {"omega", "c", "k"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"omega", "c"}; } }

    public int Seed { get; private set; }

    public Feynman28() : this((int) DateTime.Now.Ticks) { }

    public Feynman28(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var omega = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 10).ToList();
      var c     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 10).ToList();

      var k = new List<double>();

      data.Add(omega);
      data.Add(c);
      data.Add(k);

      for (var i = 0; i < omega.Count; i++) {
        var res = omega[i] / c[i];
        k.Add(res);
      }

      return data;
    }
  }
}