using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman29 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.29.16 sqrt(x1**2+x2**2-2*x1*x2*cos(theta1-theta2))"; } }

    protected override string TargetVariable { get { return "x"; } }
    protected override string[] VariableNames { get { return new[] {"x1", "x2", "theta1", "theta2", "x"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"x1", "x2", "theta1", "theta2"}; } }

    public int Seed { get; private set; }

    public Feynman29() : this((int) DateTime.Now.Ticks) { }

    public Feynman29(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data   = new List<List<double>>();
      var x1     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var x2     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var x = new List<double>();

      data.Add(x1);
      data.Add(x2);
      data.Add(theta1);
      data.Add(theta2);
      data.Add(x);

      for (var i = 0; i < x1.Count; i++) {
        var res = Math.Sqrt(Math.Pow(x1[i], 2) + Math.Pow(x2[i], 2) - 2 * x1[i] * x2[i] * Math.Cos(theta1[i] - theta2[i]));
        x.Add(res);
      }

      return data;
    }
  }
}