using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman30 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.30.3 Int_0*sin(n*theta/2)**2/sin(theta/2)**2"; } }

    protected override string TargetVariable { get { return "Int"; } }
    protected override string[] VariableNames { get { return new[] {"Int_0", "theta", "n", "Int"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"Int_0", "theta", "n"}; } }

    public int Seed { get; private set; }

    public Feynman30() : this((int) DateTime.Now.Ticks) { }

    public Feynman30(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var Int_0 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var theta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var n     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Int = new List<double>();

      data.Add(Int_0);
      data.Add(theta);
      data.Add(n);
      data.Add(Int);

      for (var i = 0; i < Int_0.Count; i++) {
        var res = Int_0[i] * Math.Pow(Math.Sin(n[i] * theta[i] / 2), 2) / Math.Pow(Math.Sin(theta[i] / 2), 2);
        Int.Add(res);
      }

      return data;
    }
  }
}