using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman31 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.30.5 arcsin(lambd/(n*d))"; } }

    protected override string TargetVariable { get { return "theta"; } }
    protected override string[] VariableNames { get { return new[] {"lambd", "d", "n", "theta"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"lambd", "d", "n"}; } }

    public int Seed { get; private set; }

    public Feynman31() : this((int) DateTime.Now.Ticks) { }

    public Feynman31(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var lambd = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var d     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 2, 5).ToList();
      var n     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var theta = new List<double>();

      data.Add(lambd);
      data.Add(d);
      data.Add(n);
      data.Add(theta);

      for (var i = 0; i < lambd.Count; i++) {
        var res = Math.Sinh(lambd[i] / (n[i] * d[i]));
        theta.Add(res);
      }

      return data;
    }
  }
}