using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman38 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.37.4 I1+I2+2*sqrt(I1*I2)*cos(delta)"; } }

    protected override string TargetVariable { get { return "Int"; } }
    protected override string[] VariableNames { get { return new[] {"I1", "I2", "delta", "Int"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"I1", "I2", "delta"}; } }

    public int Seed { get; private set; }

    public Feynman38() : this((int) DateTime.Now.Ticks) { }

    public Feynman38(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var I1    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var I2    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var delta = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var Int = new List<double>();

      data.Add(I1);
      data.Add(I2);
      data.Add(delta);
      data.Add(Int);

      for (var i = 0; i < I1.Count; i++) {
        var res = I1[i] + I2[i] + 2 * Math.Sqrt(I1[i] * I2[i]) * Math.Cos(delta[i]);
        Int.Add(res);
      }

      return data;
    }
  }
}