﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman4 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.8.14 d = sqrt((x2-x1)^2+(y2-y1)^2)"; } }

 protected override string TargetVariable { get { return "d"; } }
    protected override string[] VariableNames { get { return new string[] { "x1", "x2", "y1", "y2", "d"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"x1", "x2", "y1", "y2"}; } }

    public int Seed { get; private set; }

    public Feynman4() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman4(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var x1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var x2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var y1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var y2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var d = new List<double>();

      data.Add(x1);
      data.Add(x2);
      data.Add(y1);
      data.Add(y2);
      data.Add(d);


      for (var i = 0; i < x1.Count; i++) {
        var res = Math.Sqrt(Math.Pow((x2[i] - x1[i]), 2) + Math.Pow((y2[i] - y1[i]), 2));
        d.Add(res);
      }


      return data;
    }
  }
}
