using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman40 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.39.1 3/2*pr*V"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"pr", "V", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"pr", "V"}; } }

    public int Seed { get; private set; }

    public Feynman40() : this((int) DateTime.Now.Ticks) { }

    public Feynman40(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var pr   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var V    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(pr);
      data.Add(V);
      data.Add(E_n);

      for (var i = 0; i < pr.Count; i++) {
        var res = 3.0 / 2 * pr[i] * V[i];
        E_n.Add(res);
      }

      return data;
    }
  }
}