using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman42 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.39.22 n*kb*T/V"; } }

    protected override string TargetVariable { get { return "pr"; } }
    protected override string[] VariableNames { get { return new[] {"n", "T", "V", "kb", "pr"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "T", "V", "kb"}; } }

    public int Seed { get; private set; }

    public Feynman42() : this((int) DateTime.Now.Ticks) { }

    public Feynman42(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var n    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var V    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var pr = new List<double>();

      data.Add(n);
      data.Add(T);
      data.Add(V);
      data.Add(kb);
      data.Add(pr);

      for (var i = 0; i < n.Count; i++) {
        var res = n[i] * kb[i] * T[i] / V[i];
        pr.Add(res);
      }

      return data;
    }
  }
}