using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman43 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.40.1 n_0*exp(-m*g*x/(kb*T))"; } }

    protected override string TargetVariable { get { return "n"; } }
    protected override string[] VariableNames { get { return new[] {"n_0", "m", "x", "T", "g", "kb", "n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n_0", "m", "x", "T", "g", "kb"}; } }

    public int Seed { get; private set; }

    public Feynman43() : this((int) DateTime.Now.Ticks) { }

    public Feynman43(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var n_0  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var m    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var x    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var g    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var n = new List<double>();

      data.Add(n_0);
      data.Add(m);
      data.Add(x);
      data.Add(T);
      data.Add(g);
      data.Add(kb);
      data.Add(n);

      for (var i = 0; i < n_0.Count; i++) {
        var res = n_0[i] * Math.Exp(-m[i] * g[i] * x[i] / (kb[i] * T[i]));
        n.Add(res);
      }

      return data;
    }
  }
}