using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman45 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.43.16 mu_drift*q*Volt/d"; } }

    protected override string TargetVariable { get { return "v"; } }
    protected override string[] VariableNames { get { return new[] {"mu_drift", "q", "Volt", "d", "v"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mu_drift", "q", "Volt", "d"}; } }

    public int Seed { get; private set; }

    public Feynman45() : this((int) DateTime.Now.Ticks) { }

    public Feynman45(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data     = new List<List<double>>();
      var mu_drift = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var q        = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var Volt     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var d        = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var v = new List<double>();

      data.Add(mu_drift);
      data.Add(q);
      data.Add(Volt);
      data.Add(d);
      data.Add(v);

      for (var i = 0; i < mu_drift.Count; i++) {
        var res = mu_drift[i] * q[i] * Volt[i] / d[i];
        v.Add(res);
      }

      return data;
    }
  }
}