using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman46 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.43.31 mob*kb*T"; } }

    protected override string TargetVariable { get { return "D"; } }
    protected override string[] VariableNames { get { return new[] {"mob", "T", "kb", "D"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"mob", "T", "kb"}; } }

    public int Seed { get; private set; }

    public Feynman46() : this((int) DateTime.Now.Ticks) { }

    public Feynman46(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var mob  = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var D = new List<double>();

      data.Add(mob);
      data.Add(T);
      data.Add(kb);
      data.Add(D);

      for (var i = 0; i < mob.Count; i++) {
        var res = mob[i] * kb[i] * T[i];
        D.Add(res);
      }

      return data;
    }
  }
}