using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman47 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.43.43 1/(gamma-1)*kb*v/A"; } }

    protected override string TargetVariable { get { return "kappa"; } }
    protected override string[] VariableNames { get { return new[] {"gamma", "kb", "A", "v", "kappa"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"gamma", "kb", "A", "v"}; } }

    public int Seed { get; private set; }

    public Feynman47() : this((int) DateTime.Now.Ticks) { }

    public Feynman47(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data  = new List<List<double>>();
      var gamma = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 2, 5).ToList();
      var kb    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var A     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v     = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var kappa = new List<double>();

      data.Add(gamma);
      data.Add(kb);
      data.Add(A);
      data.Add(v);
      data.Add(kappa);

      for (var i = 0; i < gamma.Count; i++) {
        var res = 1.0 / (gamma[i] - 1) * kb[i] * v[i] / A[i];
        kappa.Add(res);
      }

      return data;
    }
  }
}