using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman48 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.44.4 n*kb*T*ln(V2/V1)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"n", "kb", "T", "V1", "V2", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"n", "kb", "T", "V1", "V2"}; } }

    public int Seed { get; private set; }

    public Feynman48() : this((int) DateTime.Now.Ticks) { }

    public Feynman48(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var n    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var kb   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var T    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var V1   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var V2   = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();

      var E_n = new List<double>();

      data.Add(n);
      data.Add(kb);
      data.Add(T);
      data.Add(V1);
      data.Add(V2);
      data.Add(E_n);

      for (var i = 0; i < n.Count; i++) {
        var res = n[i] * kb[i] * T[i] * Math.Log(V2[i] / V1[i]);
        E_n.Add(res);
      }

      return data;
    }
  }
}