﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman5 : FeynmanDescriptor{
 public override string Name { get { return "Feynman I.9.18 F = G*m1*m2/((x2-x1)^2+(y2-y1)^2+(z2-z1)^2)"; } }

 protected override string TargetVariable { get { return "F"; } }
    protected override string[] VariableNames { get { return new string[] { "m1", "m2", "G", "x1", "x2", "y1", "y2", "z1", "z2", "F"}; } }
    protected override string[] AllowedInputVariables { get { return new string[] {"m1", "m2", "G", "x1", "x2", "y1", "y2", "z1", "z2"}; } }

    public int Seed { get; private set; }

    public Feynman5() : this((int)System.DateTime.Now.Ticks) { }

    public Feynman5(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint)Seed);

      var data = new List<List<double>>();
      var m1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var m2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var G = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var x1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 4).ToList();
      var x2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var y1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 4).ToList();
      var y2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var z1 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 4).ToList();
      var z2 = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();

      var F = new List<double>();

      data.Add(m1);
      data.Add(m2);
      data.Add(G);
      data.Add(x1);
      data.Add(x2);
      data.Add(y1);
      data.Add(y2);
      data.Add(z1);
      data.Add(z2);
      data.Add(F);

      for (var i = 0; i < x1.Count; i++) {
        var res = G[i] * m1[i] * m2[i] / (Math.Pow(x2[i] - x1[i], 2) + Math.Pow(y2[i] - y1[i], 2) + Math.Pow(z2[i] - z1[i], 2));
        F.Add(res);
      }


      return data;
    }
  }
}
