using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Random;

namespace HeuristicLab.Problems.Instances.DataAnalysis {
  public class Feynman50 : FeynmanDescriptor {
    public override string Name { get { return "Feynman I.48.2 m*c**2/sqrt(1-v**2/c**2)"; } }

    protected override string TargetVariable { get { return "E_n"; } }
    protected override string[] VariableNames { get { return new[] {"m", "v", "c", "E_n"}; } }
    protected override string[] AllowedInputVariables { get { return new[] {"m", "v", "c"}; } }

    public int Seed { get; private set; }

    public Feynman50() : this((int) DateTime.Now.Ticks) { }

    public Feynman50(int seed) {
      Seed = seed;
    }

    protected override List<List<double>> GenerateValues() {
      var rand = new MersenneTwister((uint) Seed);

      var data = new List<List<double>>();
      var m    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 5).ToList();
      var v    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 1, 2).ToList();
      var c    = ValueGenerator.GenerateUniformDistributedValues(rand.Next(), TestPartitionEnd, 3, 10).ToList();

      var E_n = new List<double>();

      data.Add(m);
      data.Add(v);
      data.Add(c);
      data.Add(E_n);

      for (var i = 0; i < m.Count; i++) {
        var res = m[i] * Math.Pow(c[i], 2) / Math.Sqrt(1 - Math.Pow(v[i], 2) / Math.Pow(c[i], 2));
        E_n.Add(res);
      }

      return data;
    }
  }
}